@@ -224,12 +224,10 @@ static void get_nss_certname(std::string& certname, Arc::Logger& logger) {
 static std::string signTypeToString(Arc::Signalgorithm alg) {
   switch(alg) {
     case Arc::SIGN_SHA1: return "sha1";
-#if (OPENSSL_VERSION_NUMBER >= 0x0090800fL)
     case Arc::SIGN_SHA224: return "sha224";
     case Arc::SIGN_SHA256: return "sha256";
     case Arc::SIGN_SHA384: return "sha384";
     case Arc::SIGN_SHA512: return "sha512";
-#endif
     default:
       break;
   }
@@ -388,7 +386,7 @@ int main(int argc, char *argv[]) {
                     use_http_comm);
 
   bool use_gsi_proxy = false;
-  options.AddOption('O', "old", istring("use GSI proxy (RFC 3820 compliant proxy is default)"), use_gsi_proxy);
+  options.AddOption('O', "old", istring("this option is not functional (old GSI proxies are not supported anymore)"), use_gsi_proxy);
 
   bool info = false;
   options.AddOption('I', "info", istring("print all information about this proxy."), info);
@@ -1209,11 +1207,16 @@ int main(int argc, char *argv[]) {
   }
   std::string policy = constraints["proxyPolicy"].empty() ? constraints["proxyPolicyFile"] : constraints["proxyPolicy"];
 
+  if (use_gsi_proxy) {
+    std::cout << Arc::IString("The old GSI proxies are not supported anymore. Please do not use -O/--old option.") << std::endl;
+    return EXIT_FAILURE;
+  }
+
   if (!myproxy_command.empty() && (Arc::lower(myproxy_command) != "put")) {
     bool res = contact_myproxy_server(myproxy_server, myproxy_command, 
       user_name, use_empty_passphrase, myproxy_period, retrievable_by_cert, 
       proxy_start, proxy_period, vomslist, vomses_path, proxy_path, usercfg, logger);
-    if (res && (Arc::lower(myproxy_command) == "get")) {
+    if (res && (Arc::lower(myproxy_command) == "get") && (!vomslist.empty())) {
       // IF the myproxy command is "Get", and voms command is given,
       // then we need to check if the proxy returned from myproxy server
       // includes VOMS AC, if not, we will use the returned proxy to
@@ -1234,7 +1237,7 @@ int main(int argc, char *argv[]) {
           Arc::Credential signer(proxy_path, proxy_path, "", "");
           std::string proxy_cert;
           create_proxy(proxy_cert, signer, policy, proxy_start, proxy_period, 
-              vomsacseq, use_gsi_proxy, keybits, signing_algorithm);
+              vomsacseq, keybits, signing_algorithm);
           write_proxy_file(proxy_path, proxy_cert);
         }
       }
@@ -1291,7 +1294,7 @@ int main(int argc, char *argv[]) {
 
     std::string proxy_cert;
     create_proxy(proxy_cert, signer, policy, proxy_start, proxy_period,      
-        vomsacseq, use_gsi_proxy, keybits, signing_algorithm);
+        vomsacseq, keybits, signing_algorithm);
 
     //If myproxy command is "Put", then the proxy path is set to /tmp/myproxy-proxy.uid.pid 
     if (Arc::lower(myproxy_command) == "put")
