@@ -34,7 +34,6 @@ using namespace Arc;
 
   typedef struct {
     DataPointHTTP *point;
-    //ClientHTTP *client;
   } HTTPInfo_t;
 
   class ChunkControl {
@@ -503,9 +502,9 @@ using namespace Arc;
           return DataStatus(DataStatus::StatError,r.getExplanation());
         }
       }
-      release_client(rurl,client.Release());
       if ((info.code != 200) && (info.code != 207)) { // 207 for multistatus response
         if (inbuf) delete inbuf; inbuf = NULL;
+        release_client(rurl,client.Release());
         if((info.code == 301) || // permanent redirection
            (info.code == 302) || // temporary redirection
            (info.code == 303) || // POST to GET redirection
@@ -530,6 +529,7 @@ using namespace Arc;
       if(inbuf) {
         XMLNode multistatus(ContentFromPayload(*inbuf));
         delete inbuf; inbuf = NULL;
+        release_client(rurl,client.Release());
         if(multistatus.Name() == "multistatus") {
           XMLNode response = multistatus["response"];
           if((bool)response) {
@@ -539,6 +539,8 @@ using namespace Arc;
             }
           }
         }
+      } else {
+        release_client(rurl,client.Release());
       }
       return DataStatus(DataStatus::StatError,"Can't process WebDAV response");
     }
@@ -581,9 +583,9 @@ using namespace Arc;
           return DataStatus(DataStatus::StatError,r.getExplanation());
         }
       }
-      release_client(rurl,client.Release());
       if ((info.code != 200) && (info.code != 207)) { // 207 for multistatus response
         if (inbuf) delete inbuf; inbuf = NULL;
+        release_client(rurl,client.Release());
         if((info.code == 301) || // permanent redirection
            (info.code == 302) || // temporary redirection
            (info.code == 303) || // POST to GET redirection
@@ -608,6 +610,7 @@ using namespace Arc;
       if(inbuf) {
         XMLNode multistatus(ContentFromPayload(*inbuf));
         delete inbuf; inbuf = NULL;
+        release_client(rurl,client.Release());
         if(multistatus.Name() == "multistatus") {
           XMLNode response = multistatus["response"];
           for(;(bool)response;++response) {
@@ -632,6 +635,8 @@ using namespace Arc;
           }
           return DataStatus::Success;
         }
+      } else {
+        release_client(rurl,client.Release());
       }
       return DataStatus(DataStatus::StatError,"Can't process WebDAV response");
     }
@@ -949,7 +954,7 @@ using namespace Arc;
     HTTPInfo_t& info = *((HTTPInfo_t*)arg);
     DataPointHTTP& point = *(info.point);
     URL client_url = point.url;
-    ClientHTTP *client = point.acquire_client(client_url);
+    AutoPointer<ClientHTTP> client(point.acquire_client(client_url));
     bool transfer_failure = false;
     int retries = 0;
     std::string path = point.CurrentLocation().FullPathURIEncoded();
@@ -964,9 +969,11 @@ using namespace Arc;
                                      &instream);
       if (!r) {
         if (instream) delete instream;
-        delete client; client = NULL;
         // Failed to transfer - retry.
         // 10 times in a row seems to be reasonable number
+        // To make it sure all retriable errors are eliminated
+        // connection is also re-established
+        client = NULL;
         // TODO: mark failure?
         if ((++retries) > 10) {
           transfer_failure = true;
@@ -975,7 +982,9 @@ using namespace Arc;
         }
         // Recreate connection
         client = point.acquire_new_client(client_url);
-        continue;
+        if(client) continue;
+        transfer_failure = true;
+        break;
       }
       if((transfer_info.code == 301) || // permanent redirection
          (transfer_info.code == 302) || // temporary redirection
@@ -984,7 +993,7 @@ using namespace Arc;
         // 305 - redirection to proxy - unhandled
         if (instream) delete instream;
         // Recreate connection now to new URL
-        point.release_client(client_url,client); client = NULL; // return client to poll
+        point.release_client(client_url,client.Release()); // return client to poll
         client_url = transfer_info.location;
         logger.msg(VERBOSE,"Redirecting to %s",transfer_info.location.str());
         client = point.acquire_client(client_url);
@@ -1044,6 +1053,7 @@ using namespace Arc;
       // End of transfer - either success or not retrying transfer of whole body
       break;
     }
+    point.release_client(client_url,client.Release());
     return !transfer_failure;
   }
 
@@ -1053,13 +1063,14 @@ using namespace Arc;
     point.transfer_lock.lock();
     point.transfer_lock.unlock();
     URL client_url = point.url;
-    ClientHTTP* client = point.acquire_client(client_url);
+    AutoPointer<ClientHTTP> client(point.acquire_client(client_url));
     bool transfer_failure = false;
     int retries = 0;
     std::string path = point.CurrentLocation().FullPathURIEncoded();
     DataStatus failure_code;
     bool partial_read_allowed = (client_url.Option("httpgetpartial","yes") == "yes");
     if(partial_read_allowed) for (;;) {
+      if(client && client->GetClosed()) client = point.acquire_client(client_url);
       if (!client) {
         transfer_failure = true;
         break;
@@ -1092,7 +1103,7 @@ using namespace Arc;
         point.buffer->is_read(transfer_handle, 0, 0);
         point.chunks->Unclaim(transfer_offset, chunk_length);
         if (inbuf) delete inbuf;
-        delete client; client = NULL;
+        client = NULL;
         // Failed to transfer chunk - retry.
         // 10 times in a row seems to be reasonable number
         // TODO: mark failure?
@@ -1103,7 +1114,9 @@ using namespace Arc;
         }
         // Recreate connection
         client = point.acquire_new_client(client_url);
-        continue;
+        if(client) continue;
+        transfer_failure = true;
+        break;
       }
       if (transfer_info.code == 416) { // EOF
         point.buffer->is_read(transfer_handle, 0, 0);
@@ -1121,7 +1134,7 @@ using namespace Arc;
         point.chunks->Unclaim(transfer_offset, chunk_length);
         if (inbuf) delete inbuf;
         // Recreate connection now to new URL
-        point.release_client(client_url,client); client = NULL;
+        point.release_client(client_url,client.Release());
         client_url = transfer_info.location;
         logger.msg(VERBOSE,"Redirecting to %s",transfer_info.location.str());
         client = point.acquire_client(client_url);
@@ -1140,6 +1153,7 @@ using namespace Arc;
         if ((transfer_info.code == 500) ||
             (transfer_info.code == 503) ||
             (transfer_info.code == 504)) {
+          // Retriable error codes
           if ((++retries) <= 10) continue;
         }
         logger.msg(VERBOSE,"HTTP failure %u - %s",transfer_info.code,transfer_info.reason);
@@ -1193,43 +1207,6 @@ using namespace Arc;
         transfer_handle = -1;
         transfer_pos = pos + l;
       }
-      /*
-      for (unsigned int n = 0;; ++n) {
-        if (!inbuf) break;
-        char *buf = inbuf->Buffer(n);
-        if (!buf) break;
-        uint64_t pos = inbuf->BufferPos(n);
-        unsigned int length = inbuf->BufferSize(n);
-        transfer_pos = inbuf->BufferPos(n) + inbuf->BufferSize(n);
-        // In general case returned chunk may be of different size than
-        // requested
-        for (; length;) {
-          if (transfer_handle == -1) {
-            // Get transfer buffer if needed
-            transfer_size = 0;
-            point.transfer_lock.unlock();
-            if (!point.buffer->for_read(transfer_handle, transfer_size,
-                                        true)) {
-              // No transfer buffer - must be failure or close initiated
-              // externally
-              point.transfer_lock.lock();
-              break;
-            }
-            point.transfer_lock.lock();
-          }
-          unsigned int l = length;
-          if (l > transfer_size) l = transfer_size;
-          char *buf_ = (*point.buffer)[transfer_handle];
-          memcpy(buf_, buf, l);
-          point.buffer->is_read(transfer_handle, l, pos);
-          point.chunks->Claim(pos, l);
-          length -= l;
-          pos += l;
-          buf += l;
-          transfer_handle = -1;
-        }
-      }
-      */
       if (transfer_handle != -1) point.buffer->is_read(transfer_handle, 0, 0);
       if (inbuf) delete inbuf;
       // If server returned chunk which is not overlaping requested one - seems
@@ -1255,7 +1232,7 @@ using namespace Arc;
       }
       point.buffer->eof_read(true);
     }
-    point.release_client(client_url,client); client = NULL;
+    point.release_client(client_url,client.Release());
     delete &info;
     point.transfer_lock.unlock();
   }
@@ -1264,7 +1241,7 @@ using namespace Arc;
     HTTPInfo_t& info = *((HTTPInfo_t*)arg);
     DataPointHTTP& point = *(info.point);
     URL client_url = point.url;
-    ClientHTTP *client = point.acquire_client(client_url);
+    AutoPointer<ClientHTTP> client(point.acquire_client(client_url));
     if (!client) return false;
     HTTPClientInfo transfer_info;
     PayloadRawInterface *response = NULL;
@@ -1286,6 +1263,7 @@ using namespace Arc;
         // at lower level.
       }
       StreamBuffer request(*point.buffer);
+      if (point.CheckSize()) request.Size(point.GetSize());
       MCC_Status r = client->process(ClientHTTPAttributes("PUT", path, attrs),
                                      &request, &transfer_info, &response);
       if (response) delete response; response = NULL;
@@ -1293,7 +1271,7 @@ using namespace Arc;
         // It is not clear how to retry if early chunks are not available anymore.
         // Let it retry at higher level.
         point.failure_code = DataStatus(DataStatus::WriteError, r.getExplanation());
-        delete client; client = NULL;
+        client = NULL;
         return false;
       }
       if (transfer_info.code == 301 || // Moved permanently
@@ -1301,7 +1279,7 @@ using namespace Arc;
           transfer_info.code == 307) { // Temporary redirection
         // Got redirection response
         // Recreate connection now to new URL
-        point.release_client(client_url,client); client = NULL;
+        point.release_client(client_url,client.Release());
         client_url = transfer_info.location;
         logger.msg(VERBOSE,"Redirecting to %s",transfer_info.location.str());
         client = point.acquire_client(client_url);
@@ -1332,12 +1310,14 @@ using namespace Arc;
       if ((transfer_info.code != 201) &&
           (transfer_info.code != 200) &&
           (transfer_info.code != 204)) {  // HTTP error
+        point.release_client(client_url,client.Release());
         point.failure_code = DataStatus(DataStatus::WriteError, point.http2errno(transfer_info.code), transfer_info.reason);
         return false;
       }
       // Looks like request passed well
       break;
     }
+    point.release_client(client_url,client.Release());
     return true;
   }
 
@@ -1347,7 +1327,7 @@ using namespace Arc;
     point.transfer_lock.lock();
     point.transfer_lock.unlock();
     URL client_url = point.url;
-    ClientHTTP *client = point.acquire_client(client_url);
+    AutoPointer<ClientHTTP> client(point.acquire_client(client_url));
     bool transfer_failure = false;
     int retries = 0;
     std::string path = client_url.FullPathURIEncoded();
@@ -1355,6 +1335,7 @@ using namespace Arc;
     DataStatus failure_code;
     // Fall through if partial PUT is not allowed
     if(!partial_write_failure) for (;;) {
+      if(client && client->GetClosed()) client = point.acquire_client(client_url);
       if (!client) {
         transfer_failure = true;
         break;
@@ -1379,7 +1360,7 @@ using namespace Arc;
                                      &response);
       if (response) delete response;
       if (!r) {
-        delete client; client = NULL;
+        client = NULL;
         // Failed to transfer chunk - retry.
         // 10 times in a row seems to be reasonable number
         // TODO: mark failure?
@@ -1447,7 +1428,7 @@ using namespace Arc;
                                          &response);
           if (response) delete response;
           if (!r) {
-            delete client; client = NULL;
+            client = NULL;
             if ((++retries) > 10) {
               point.failure_code = DataStatus(DataStatus::WriteError, r.getExplanation());
               point.buffer->error_write(true);
@@ -1472,7 +1453,7 @@ using namespace Arc;
         }
       }
     }
-    point.release_client(client_url,client); client = NULL;
+    point.release_client(client_url,client.Release());
     delete &info;
     point.transfer_lock.unlock();
   }
@@ -1483,6 +1464,7 @@ using namespace Arc;
     if(url.Port() != this->url.Port()) return false;
     this->url = url;
     if(triesleft < 1) triesleft = 1;
+    ResetMeta();
     return true;
   }
 
@@ -1521,6 +1503,7 @@ using namespace Arc;
 
   void DataPointHTTP::release_client(const URL& curl, ClientHTTP* client) {
     if(!client) return;
+    if(client->GetClosed()) { delete client; return; }
     std::string key = curl.ConnectionURL();
     //if(!*client) return;
     clients_lock.lock();
