@@ -381,6 +381,46 @@ namespace Arc {
         return ss.str();
       }
 
+    case UserExtTime:
+      {
+        tm tmtime;
+        localtime_r(&gtime, &tmtime);
+
+        std::stringstream ss;
+
+        ss << std::setfill('0');
+
+        ss << std::setw(4) << tmtime.tm_year + 1900 << '-'
+           << std::setw(2) << tmtime.tm_mon + 1 << '-'
+           << std::setw(2) << tmtime.tm_mday << ' '
+           << std::setw(2) << tmtime.tm_hour << ':'
+           << std::setw(2) << tmtime.tm_min << ':'
+           << std::setw(2) << tmtime.tm_sec << '.'
+           << std::setw(6) << gnano/1000;
+
+        return ss.str();
+      }
+
+    case ElasticTime:
+      {
+        tm tmtime;
+        localtime_r(&gtime, &tmtime);
+
+        std::stringstream ss;
+
+        ss << std::setfill('0');
+
+        ss << std::setw(4) << tmtime.tm_year + 1900 << '-'
+           << std::setw(2) << tmtime.tm_mon + 1 << '-'
+           << std::setw(2) << tmtime.tm_mday << ' '
+           << std::setw(2) << tmtime.tm_hour << ':'
+           << std::setw(2) << tmtime.tm_min << ':'
+           << std::setw(2) << tmtime.tm_sec << '.'
+           << std::setw(3) << gnano/1000000;
+
+        return ss.str();
+      }
+
     case MDSTime:
       {
         tm tmtime;
@@ -806,6 +846,10 @@ namespace Arc {
 
     std::stringstream ss;
 
+    /*
+     Size of year, month and even day is variable.
+     To avoid ambiguity let's keep only time parts.
+ 
     if (remain >= Time::YEAR) {
       ss << remain / Time::YEAR << " "
          << FindNTrans("year", "years", remain / Time::YEAR);
@@ -823,6 +867,7 @@ namespace Arc {
          << FindNTrans("day", "days", remain / Time::DAY);
       remain %= Time::DAY;
     }
+    */
     if (remain >= Time::HOUR) {
       if (remain != seconds) ss << " ";
       ss << remain / Time::HOUR << " "
@@ -851,6 +896,10 @@ namespace Arc {
     std::stringstream ss;
 
     ss << 'P';
+    /*
+     Size of year, month and even day is variable.
+     To avoid ambiguity let's keep only time parts.
+ 
     if (remain >= Time::YEAR) {
       ss << remain / Time::YEAR << 'Y';
       remain %= Time::YEAR;
@@ -863,18 +912,22 @@ namespace Arc {
       ss << remain / Time::DAY << 'D';
       remain %= Time::DAY;
     }
-    if (remain)
+    */
+    if (remain) {
       ss << 'T';
-    if (remain >= Time::HOUR) {
-      ss << remain / Time::HOUR << 'H';
-      remain %= Time::HOUR;
+      if (remain >= Time::HOUR) {
+        ss << remain / Time::HOUR << 'H';
+        remain %= Time::HOUR;
+      }
+      if (remain >= 60) {
+        ss << remain / 60 << 'M';
+        remain %= 60;
+      }
+      if (remain >= 1)
+        ss << remain << 'S';
+    } else {
+      ss << "T0S";
     }
-    if (remain >= 60) {
-      ss << remain / 60 << 'M';
-      remain %= 60;
-    }
-    if (remain >= 1)
-      ss << remain << 'S';
 
     return ss.str();
   }
