@@ -74,7 +74,10 @@ bool FileCopy(const std::string& source_path,const std::string& destination_path
   struct stat st;
   int source_handle = ::open(source_path.c_str(),O_RDONLY,0);
   if(source_handle == -1) return false;
-  if(::fstat(source_handle,&st) != 0) return false;
+  if(::fstat(source_handle,&st) != 0) {
+    ::close(source_handle);
+    return false;
+  }
   int destination_handle = ::open(destination_path.c_str(),O_WRONLY | O_CREAT | O_TRUNC,st.st_mode);
   if(destination_handle == -1) {
     ::close(source_handle);
@@ -219,14 +222,13 @@ bool FileCreate(const std::string& filename, const std::string& data, uid_t uid,
 #else
   if(mode == 0) mode = S_IRUSR|S_IWUSR;
 #endif
-  char* tempfile = strdup(std::string(filename+".XXXXXX").c_str());
-  int h = ::mkstemp(tempfile);
-  if(h == -1) { free(tempfile); return false; }
-  if(!write_all(h,data.c_str(),data.length())) { ::close(h); free(tempfile); return false; }
+  std::string tempfile = filename+".XXXXXX";
+  int h = ::mkstemp(const_cast<char*>(tempfile.c_str()));
+  if(h == -1) return false;
+  if(!write_all(h,data.c_str(),data.length())) { ::close(h); return false; }
   ::close(h);
-  if(chmod(tempfile, mode) != 0) { unlink(tempfile); free(tempfile); return false; }
-  if(rename(tempfile, filename.c_str()) != 0) { unlink(tempfile); free(tempfile); return false; }
-  free(tempfile);
+  if(chmod(tempfile.c_str(), mode) != 0) { unlink(tempfile.c_str()); return false; }
+  if(rename(tempfile.c_str(), filename.c_str()) != 0) { unlink(tempfile.c_str()); return false; }
   return true;
 }
 
