@@ -318,8 +318,8 @@ namespace Arc {
   }
 
   bool UserConfig::JobListType(const std::string& type) {
-    if (type != "XML" && type != "BDB") {
-      logger.msg(WARNING, "Unsupported job list type '%s', using 'BDB'. Supported types are: BDB, XML.", type);
+    if (type != "XML" && type != "BDB" && type != "SQLITE") {
+      logger.msg(WARNING, "Unsupported job list type '%s', using 'BDB'. Supported types are: BDB, SQLITE, XML.", type);
       joblisttype = "BDB";
       return true;
     }
@@ -404,7 +404,6 @@ namespace Arc {
       }
     }
 
-    if(has_proxy) require = false;
     // Should we really handle them in pairs
     std::string cert_path = GetEnv("X509_USER_CERT");
     std::string key_path = GetEnv("X509_USER_KEY");
@@ -412,16 +411,16 @@ namespace Arc {
       certificatePath = cert_path;
       file_test_status fts;
       if (test && ((fts = user_file_test(certificatePath, user)) != file_test_success)) {
-        certificate_file_error_report(fts,require,certificatePath,logger);
-        if(require) {
+        certificate_file_error_report(fts,!has_proxy,certificatePath,logger);
+        if(!has_proxy) {
           res = false;
         }
         certificatePath.clear();
       }
       keyPath = key_path;
       if (test && ((fts = private_file_test(keyPath, user)) != file_test_success)) {
-        key_file_error_report(fts,require,keyPath,logger);
-        if(require) {
+        key_file_error_report(fts,!has_proxy,keyPath,logger);
+        if(!has_proxy) {
           res = false;
         }
         keyPath.clear();
@@ -429,15 +428,15 @@ namespace Arc {
     } else if (!certificatePath.empty() && !keyPath.empty()) {
       file_test_status fts;
       if (test && ((fts = user_file_test(certificatePath, user)) != file_test_success)) {
-        certificate_file_error_report(fts,require,certificatePath,logger);
-        if(require) {
+        certificate_file_error_report(fts,!has_proxy,certificatePath,logger);
+        if(!has_proxy) {
           res = false;
         }
         certificatePath.clear();
       }
       if (test && ((fts = private_file_test(keyPath, user)) != file_test_success)) {
-        key_file_error_report(fts,require,keyPath,logger);
-        if(require) {
+        key_file_error_report(fts,!has_proxy,keyPath,logger);
+        if(!has_proxy) {
           res = false;
         }
         keyPath.clear();
@@ -446,8 +445,8 @@ namespace Arc {
       //If only certificatePath provided, then it could be a pkcs12 file
       file_test_status fts;
       if (test && ((fts = user_file_test(certificatePath, user)) != file_test_success)) {
-        certificate_file_error_report(fts,require,certificatePath,logger);
-        if(require) {
+        certificate_file_error_report(fts,!has_proxy,certificatePath,logger);
+        if(!has_proxy) {
           res = false;
         }
         certificatePath.clear();
@@ -457,8 +456,8 @@ namespace Arc {
       certificatePath = cert_path;
       file_test_status fts;
       if (test && ((fts = user_file_test(cert_path, user)) != file_test_success)) {
-        certificate_file_error_report(fts,require,cert_path,logger);
-        if(require) {
+        certificate_file_error_report(fts,!has_proxy,cert_path,logger);
+        if(!has_proxy) {
           res = false;
         }
         cert_path.clear();
@@ -501,21 +500,23 @@ namespace Arc {
       }
       if (it == search_paths.end() && !has_proxy) {
         logger.msg(VERBOSE, "Certificate and key ('%s' and '%s') not found in any of the paths: %s", "usercert.pem", "userkey.pem", tried_paths);
-        logger.msg(require?WARNING:VERBOSE,
+        logger.msg(!has_proxy?WARNING:VERBOSE,
           "If the proxy or certificate/key does exist, you can manually specify the locations via environment variables "
           "'%s'/'%s' or '%s', or the '%s'/'%s' or '%s' attributes in the client configuration file (e.g. '%s')",
           "X509_USER_CERT", "X509_USER_KEY", "X509_USER_PROXY", "certificatepath", "proxypath", "keypath", "~/.arc/client.conf");
       }
-      if((certificatePath.empty() || keyPath.empty()) && require) {
+      if((certificatePath.empty() || keyPath.empty()) && !has_proxy) {
         res = false;
       }
     }
 
     if(!noca) {
       std::string ca_dir = GetEnv("X509_CERT_DIR");
+      //std::cerr<<"-- ca_dir = "<<ca_dir<<std::endl;
       if (!ca_dir.empty()) {
         caCertificatesDirectory = ca_dir;
         if (test && !dir_test(caCertificatesDirectory)) {
+          //std::cerr<<"-- ca_dir test failed"<<std::endl;
           if(require) {
             logger.msg(WARNING, "Can not access CA certificates directory: %s. The certificates will not be verified.", caCertificatesDirectory);
             res = false;
@@ -523,7 +524,9 @@ namespace Arc {
           caCertificatesDirectory.clear();
         }
       } else if (!caCertificatesDirectory.empty()) {
+        //std::cerr<<"-- caCertificatesDirectory = "<<caCertificatesDirectory<<std::endl;
         if (test && !dir_test(caCertificatesDirectory)) {
+          //std::cerr<<"-- caCertificatesDirectory test failed"<<std::endl;
           if(require) {
             logger.msg(WARNING, "Can not access CA certificate directory: %s. The certificates will not be verified.", caCertificatesDirectory);
             res = false;
@@ -532,16 +535,22 @@ namespace Arc {
         }
       } else {
         caCertificatesDirectory = home_path+G_DIR_SEPARATOR_S+".arc"+G_DIR_SEPARATOR_S+"certificates";
+        //std::cerr<<"-- option 1 = "<<caCertificatesDirectory<<std::endl;
         if (test && !dir_test(caCertificatesDirectory)) {
           caCertificatesDirectory = home_path+G_DIR_SEPARATOR_S+".globus"+G_DIR_SEPARATOR_S+"certificates";
+          //std::cerr<<"-- option 2 = "<<caCertificatesDirectory<<std::endl;
           if (!dir_test(caCertificatesDirectory)) {
             caCertificatesDirectory = ArcLocation::Get()+G_DIR_SEPARATOR_S+"etc"+G_DIR_SEPARATOR_S+"certificates";
+            //std::cerr<<"-- option 3 = "<<caCertificatesDirectory<<std::endl;
             if (!dir_test(caCertificatesDirectory)) {
               caCertificatesDirectory = ArcLocation::Get()+G_DIR_SEPARATOR_S+"etc"+G_DIR_SEPARATOR_S+"grid-security"+G_DIR_SEPARATOR_S+"certificates";
+              //std::cerr<<"-- option 4 = "<<caCertificatesDirectory<<std::endl;
               if (!dir_test(caCertificatesDirectory)) {
                 caCertificatesDirectory = ArcLocation::Get()+G_DIR_SEPARATOR_S+"share"+G_DIR_SEPARATOR_S+"certificates";
+                //std::cerr<<"-- option 5 = "<<caCertificatesDirectory<<std::endl;
                 if (!dir_test(caCertificatesDirectory)) {
                   caCertificatesDirectory = std::string(G_DIR_SEPARATOR_S)+"etc"+G_DIR_SEPARATOR_S+"grid-security"+G_DIR_SEPARATOR_S+"certificates";
+                  //std::cerr<<"-- option 6 = "<<caCertificatesDirectory<<std::endl;
                   if (!dir_test(caCertificatesDirectory)) {
                     if(require) {
                       logger.msg(WARNING,
