@@ -17,6 +17,17 @@
 
 namespace Arc {
 
+  #define SECURITY_IS_SSL(sec) ( \
+    (sec == TLSSec)   || \
+    (sec == TLS10Sec) || (sec == TLS11Sec)  || (sec == TLS12Sec) || \
+    (sec == SSL3Sec)  || \
+    (sec == DTLSSec)  || (sec == DTLS10Sec) || (sec ==DTLS12Sec) \
+  )
+
+  #define SECURITY_IS_GSI(sec) ( \
+    (sec == GSISec) || (sec == GSIIOSec) \
+  )
+
   Logger ClientInterface::logger(Logger::getRootLogger(), "ClientInterface");
 
   static void xml_add_element(XMLNode xml, XMLNode element) {
@@ -140,25 +151,40 @@ namespace Arc {
     if(timeout >=  0) comp.NewChild("Timeout") = tostring(timeout);
     if(no_delay) comp.NewChild("NoDelay") = "true";
 
-    if ((sec.sec == TLSSec) || (sec.sec == SSL3Sec)) {
+    if (SECURITY_IS_SSL(sec.sec)) {
       comp = ConfigMakeComponent(xmlcfg["Chain"], "tls.client", "tls", "tcp");
       if (!cfg.key.empty()) comp.NewChild("KeyPath") = cfg.key;
       if (!cfg.cert.empty()) comp.NewChild("CertificatePath") = cfg.cert;
       if (!cfg.proxy.empty()) comp.NewChild("ProxyPath") = cfg.proxy;
       if (!cfg.credential.empty()) comp.NewChild("Credential") = cfg.credential;
       if (!cfg.cafile.empty()) comp.NewChild("CACertificatePath") = cfg.cafile;
-      if (!cfg.cadir.empty()) comp.NewChild("CACertificatesDir") = cfg.cadir;
+      if (!cfg.cadir.empty()) {
+        XMLNode cadir = comp.NewChild("CACertificatesDir");
+        cadir = cfg.cadir;
+        cadir.NewAttribute("PolicyGlobus") = "true";
+      };
       comp.NewAttribute("entry") = "tls";
       if (sec.sec == SSL3Sec) comp.NewChild("Handshake") = "SSLv3";
+      else if (sec.sec == TLS10Sec) comp.NewChild("Handshake") = "TLSv1.0";
+      else if (sec.sec == TLS11Sec) comp.NewChild("Handshake") = "TLSv1.1";
+      else if (sec.sec == TLS12Sec) comp.NewChild("Handshake") = "TLSv1.2";
+      else if (sec.sec == DTLSSec) comp.NewChild("Handshake") = "DTLS";
+      else if (sec.sec == DTLS10Sec) comp.NewChild("Handshake") = "DTLSv1.0";
+      else if (sec.sec == DTLS12Sec) comp.NewChild("Handshake") = "DTLSv1.2";
+      else comp.NewChild("Handshake") = "TLS"; // also default
     }
-    else if ((sec.sec == GSISec) || (sec.sec == GSIIOSec)) {
+    else if (SECURITY_IS_GSI(sec.sec)) {
       comp = ConfigMakeComponent(xmlcfg["Chain"], "tls.client", "gsi", "tcp");
       if (!cfg.key.empty()) comp.NewChild("KeyPath") = cfg.key;
       if (!cfg.cert.empty()) comp.NewChild("CertificatePath") = cfg.cert;
       if (!cfg.proxy.empty()) comp.NewChild("ProxyPath") = cfg.proxy;
       if (!cfg.credential.empty()) comp.NewChild("Credential") = cfg.credential;
       if (!cfg.cafile.empty()) comp.NewChild("CACertificatePath") = cfg.cafile;
-      if (!cfg.cadir.empty()) comp.NewChild("CACertificatesDir") = cfg.cadir;
+      if (!cfg.cadir.empty()) {
+        XMLNode cadir = comp.NewChild("CACertificatesDir");
+        cadir = cfg.cadir;
+        cadir.NewAttribute("PolicyGlobus") = "true";
+      };
       if (sec.sec == GSISec) {
         comp.NewChild("GSI") = "globus";
       } else {
@@ -219,11 +245,11 @@ namespace Arc {
 
     if (repmsg.Payload() != NULL) {
       try {
-        *response =
-          dynamic_cast<PayloadStreamInterface*>(repmsg.Payload());
+        *response = dynamic_cast<PayloadStreamInterface*>(repmsg.Payload());
+        repmsg.Payload(NULL);
       } catch (std::exception&) {
-        delete repmsg.Payload();
       }
+      delete repmsg.Payload();
     }
     return r;
   }
@@ -254,21 +280,21 @@ namespace Arc {
 
     if (repmsg.Payload() != NULL) {
       try {
-        *response =
-          dynamic_cast<PayloadStreamInterface*>(repmsg.Payload());
+        *response = dynamic_cast<PayloadStreamInterface*>(repmsg.Payload());
+        repmsg.Payload(NULL);
       } catch (std::exception&) {
-        delete repmsg.Payload();
       }
+      delete repmsg.Payload();
     }
     return r;
   }
 
   void ClientTCP::AddSecHandler(XMLNode handlercfg, TCPSec sec, const std::string& libname, const std::string& libpath) {
-    if ((sec.sec == TLSSec) || (sec.sec == SSL3Sec)) {
+    if (SECURITY_IS_SSL(sec.sec)) {
       ClientInterface::AddSecHandler(
         ConfigFindComponent(xmlcfg["Chain"], "tls.client", "tls"),
         handlercfg);
-    } else if ((sec.sec == GSISec) || (sec.sec == GSIIOSec)) {
+    } else if (SECURITY_IS_GSI(sec.sec)) {
       ClientInterface::AddSecHandler(
         ConfigFindComponent(xmlcfg["Chain"], "tls.client", "gsi"),
         handlercfg);
@@ -322,6 +348,7 @@ namespace Arc {
     TCPSec sec;
     if(url.Protocol() == "https") {
       if(url.Option("protocol") == "ssl3") {
+        // TODO: Other options?
         sec.sec = SSL3Sec;
       } else {
         sec.sec = TLSSec;
@@ -361,11 +388,12 @@ namespace Arc {
                 url.Option("tcpnodelay") == "yes"),
       http_entry(NULL),
       default_url(url),
-      relative_uri(false),
-      sec(http_url_to_sec(url)) {
+      relative_uri(url.Option("relativeuri") == "yes"),
+      sec(http_url_to_sec(url)),
+      closed(false) {
     XMLNode comp = ConfigMakeComponent(xmlcfg["Chain"], "http.client", "http",
-                     ((sec.sec == TLSSec) || (sec.sec == SSL3Sec)) ? "tls" :
-                     ((sec.sec == GSISec) || (sec.sec == GSIIOSec)) ? "gsi" : "tcp");
+                     (SECURITY_IS_SSL(sec.sec)) ? "tls" :
+                     (SECURITY_IS_GSI(sec.sec)) ? "gsi" : "tcp");
     comp.NewAttribute("entry") = "http";
     comp.NewChild("Method") = "POST"; // Override using attributes if needed
     comp.NewChild("Endpoint") = url.str(true); // Override using attributes if needed
@@ -440,9 +468,10 @@ namespace Arc {
     if (mresp != NULL) {
       try {
         *response = dynamic_cast<PayloadRawInterface*>(mresp);
+        mresp = NULL;
       } catch (std::exception&) {
-        delete mresp;
       }
+      delete mresp;
     }
     return r;
   }
@@ -456,6 +485,7 @@ namespace Arc {
                          MessagePayload **response) {
     *response = NULL;
     MCC_Status r;
+    if (closed) return r;
     if (!(r=Load())) return r;
     if (!http_entry) return MCC_Status(GENERIC_ERROR,"COMMUNICATION","MCC chain loading produced no entry point");
     MessageAttributes attributes_req;
@@ -526,6 +556,7 @@ namespace Arc {
     for(AttributeIterator i = repmsg.Attributes()->getAll();i.hasMore();++i) {
       info->headers.insert(std::pair<std::string, std::string>(i.key(), *i));
     }
+    closed = (repmsg.Attributes()->get("HTTP:KEEPALIVE") != "TRUE");
     *response = repmsg.Payload();
     return r;
   }
@@ -540,9 +571,10 @@ namespace Arc {
     if (mresp != NULL) {
       try {
         *response = dynamic_cast<PayloadRawInterface*>(mresp);
+        mresp = NULL;
       } catch (std::exception&) {
-        delete mresp;
       }
+      delete mresp;
     }
     return r;
   }
@@ -557,9 +589,10 @@ namespace Arc {
     if (mresp != NULL) {
       try {
         *response = dynamic_cast<PayloadRawInterface*>(mresp);
+        mresp = NULL;
       } catch (std::exception&) {
-        delete mresp;
       }
+      delete mresp;
     }
     return r;
   }
@@ -574,9 +607,10 @@ namespace Arc {
     if (mresp != NULL) {
       try {
         *response = dynamic_cast<PayloadStreamInterface*>(mresp);
+        mresp = NULL;
       } catch (std::exception&) {
-        delete mresp;
       }
+      delete mresp;
     }
     return r;
   }
@@ -591,9 +625,10 @@ namespace Arc {
     if (mresp != NULL) {
       try {
         *response = dynamic_cast<PayloadStreamInterface*>(mresp);
+        mresp = NULL;
       } catch (std::exception&) {
-        delete mresp;
       }
+      delete mresp;
     }
     return r;
   }
@@ -687,9 +722,10 @@ namespace Arc {
     if (repmsg.Payload() != NULL) {
       try {
         *response = dynamic_cast<PayloadSOAP*>(repmsg.Payload());
+        repmsg.Payload(NULL);
       } catch (std::exception&) {
-        delete repmsg.Payload();
       }
+      delete repmsg.Payload();
     }
     return r;
   }
@@ -715,9 +751,10 @@ namespace Arc {
     if (repmsg.Payload() != NULL) {
       try {
         *response = dynamic_cast<PayloadSOAP*>(repmsg.Payload());
+        repmsg.Payload(NULL);
       } catch (std::exception&) {
-        delete repmsg.Payload();
       }
+      delete repmsg.Payload();
     }
     return r;
   }
