@@ -297,7 +297,6 @@ class cred_info_t {
 
 static bool get_cred_info(const std::string& str,cred_info_t& info) {
   // It shold use Credential class. But so far keeping dependencies simple.
-  char buf[256];
   bool r = false;
   X509* cert = NULL;
   STACK_OF(X509)* cert_sk = NULL;
@@ -308,12 +307,20 @@ static bool get_cred_info(const std::string& str,cred_info_t& info) {
     info.strength=0;
     X509* c = cert;
     for(int idx = 0;;++idx) {
-      buf[0]=0;
-      X509_NAME_oneline(X509_get_issuer_name(c),buf,sizeof(buf));
-      info.ca = buf;
-      buf[0]=0;
-      X509_NAME_oneline(X509_get_subject_name(c),buf,sizeof(buf));
-      info.identity=buf;
+      char* buf = X509_NAME_oneline(X509_get_issuer_name(c),NULL,0);
+      if(buf) {
+        info.ca = buf;
+        OPENSSL_free(buf); buf = NULL;
+      } else {
+        info.ca = "";
+      }
+      buf = X509_NAME_oneline(X509_get_subject_name(c),NULL,0);
+      if(buf) {
+        info.identity=buf;
+        OPENSSL_free(buf); buf = NULL;
+      } else {
+        info.identity="";
+      }
       Time from = asn1_to_time(X509_get_notBefore(c));
       Time till = asn1_to_time(X509_get_notAfter(c));
       if(from != Time(Time::UNDEFINED)) {
@@ -326,11 +333,7 @@ static bool get_cred_info(const std::string& str,cred_info_t& info) {
           info.valid_till = till;
         };
       };
-#ifdef HAVE_OPENSSL_PROXY
       if(X509_get_ext_by_NID(cert,NID_proxyCertInfo,-1) < 0) break;
-#else
-      break;
-#endif
       if(idx >= sk_X509_num(cert_sk)) break;
       c = sk_X509_value(cert_sk,idx);
     };
@@ -365,16 +368,6 @@ const std::string& DelegationConsumer::ID(void) {
 }
 
 /*
-#ifdef HAVE_OPENSSL_OLDRSA
-static void progress_cb(int p, int, void*) {
-  char c='*';
-  if (p == 0) c='.';
-  if (p == 1) c='+';
-  if (p == 2) c='*';
-  if (p == 3) c='\n';
-  std::cerr<<c;
-}
-#else
 static int progress_cb(int p, int, BN_GENCB*) {
   char c='*';
   if (p == 0) c='.';
@@ -384,7 +377,6 @@ static int progress_cb(int p, int, BN_GENCB*) {
   std::cerr<<c;
   return 1;
 }
-#endif
 */
 
 static int ssl_err_cb(const char *str, size_t len, void *u) {
@@ -442,19 +434,6 @@ bool DelegationConsumer::Restore(const std::string& content) {
 bool DelegationConsumer::Generate(void) {
   bool res = false;
   int num = 1024;
-#ifdef HAVE_OPENSSL_OLDRSA
-  unsigned long bn = RSA_F4;
-  //RSA *rsa=RSA_generate_key(num,bn,&progress_cb,NULL);
-  RSA *rsa=RSA_generate_key(num,bn,NULL,NULL);
-  if(rsa) {
-    if(key_) RSA_free((RSA*)key_);
-    key_=rsa; rsa=NULL; res=true;
-  } else {
-    LogError();
-    std::cerr<<"RSA_generate_key failed"<<std::endl;
-  };
-  if(rsa) RSA_free(rsa);
-#else
   //BN_GENCB cb;
   BIGNUM *bn = BN_new();
   RSA *rsa = RSA_new();
@@ -480,7 +459,6 @@ bool DelegationConsumer::Generate(void) {
   };
   if(bn) BN_free(bn);
   if(rsa) RSA_free(rsa);
-#endif
   return res;
 }
 
@@ -536,7 +514,6 @@ bool DelegationConsumer::Acquire(std::string& content, std::string& identity) {
   X509 *cert = NULL;
   STACK_OF(X509) *cert_sk = NULL;
   bool res = false;
-  char buf[100];
   std::string subject;
 
   if(!key_) return false;
@@ -545,14 +522,16 @@ bool DelegationConsumer::Acquire(std::string& content, std::string& identity) {
 
   content.resize(0);
   if(!x509_to_string(cert,content)) goto err;
-
-  X509_NAME_oneline(X509_get_subject_name(cert),buf,sizeof(buf));
-  subject=buf;
-#ifdef HAVE_OPENSSL_PROXY
+  {
+    char* buf = X509_NAME_oneline(X509_get_subject_name(cert),NULL,0);
+    if(buf) {
+      subject=buf;
+      OPENSSL_free(buf);
+    };
+  };
   if(X509_get_ext_by_NID(cert,NID_proxyCertInfo,-1) < 0) {
     identity=subject;
   };
-#endif
 
   if(!x509_to_string((RSA*)key_,content)) goto err;
   if(cert_sk) {
@@ -561,13 +540,13 @@ bool DelegationConsumer::Acquire(std::string& content, std::string& identity) {
       if(!v) goto err;
       if(!x509_to_string(v,content)) goto err;
       if(identity.empty()) {
-        memset(buf,0,100);
-        X509_NAME_oneline(X509_get_subject_name(v),buf,sizeof(buf));
-#ifdef HAVE_OPENSSL_PROXY
         if(X509_get_ext_by_NID(v,NID_proxyCertInfo,-1) < 0) {
-          identity=buf;
+          char* buf = X509_NAME_oneline(X509_get_subject_name(v),NULL,0);
+          if(buf) {
+            identity=buf;
+            OPENSSL_free(buf);
+          };
         };
-#endif
       };
     };
   };
@@ -658,7 +637,6 @@ DelegationProvider::~DelegationProvider(void) {
 }
 
 std::string DelegationProvider::Delegate(const std::string& request,const DelegationRestrictions& restrictions) {
-#ifdef HAVE_OPENSSL_PROXY
   X509 *cert = NULL;
   X509_REQ *req = NULL;
   BIO* in = NULL;
@@ -797,9 +775,11 @@ std::string DelegationProvider::Delegate(const std::string& request,const Delega
         (PROXY_CERT_INFO_EXTENSION*)X509_get_ext_d2i((X509*)cert_,NID_proxyCertInfo,NULL,NULL);
     if(pci) {
       if(pci->proxyPolicy && pci->proxyPolicy->policyLanguage) {
-        char* buf = new char[256];
-        int l = OBJ_obj2txt(buf,255,pci->proxyPolicy->policyLanguage,1);
+        int const bufSize = 255;
+        char* buf = new char[bufSize+1];
+        int l = OBJ_obj2txt(buf,bufSize,pci->proxyPolicy->policyLanguage,1);
         if(l > 0) {
+          if(l > bufSize) l=bufSize;
           buf[l] = 0;
           if(strcmp(GLOBUS_LIMITED_PROXY_OID,buf) == 0) {
             // Gross hack for globus. If Globus marks own proxy as limited
@@ -810,6 +790,7 @@ std::string DelegationProvider::Delegate(const std::string& request,const Delega
             obj=OBJ_txt2obj(GLOBUS_LIMITED_PROXY_OID,1);
           };
         };
+        delete[] buf;
       };
       PROXY_CERT_INFO_EXTENSION_free(pci);
     };
@@ -917,9 +898,6 @@ err:
   if(subject) X509_NAME_free(subject);
   if(policy_string) ASN1_OCTET_STRING_free(policy_string);
   return res;
-#else
-  return "";
-#endif
 }
 
 void DelegationProvider::LogError(void) {
