@@ -29,13 +29,12 @@ DelegationCollector::~DelegationCollector(void) {
 //       Alternatively behavior may be configurable.
 static bool get_proxy_policy(X509* cert,DelegationMultiSecAttr* sattr) {
   bool result = false;
-#ifdef HAVE_OPENSSL_PROXY
   PROXY_CERT_INFO_EXTENSION *pci = (PROXY_CERT_INFO_EXTENSION*)X509_get_ext_d2i(cert,NID_proxyCertInfo,NULL,NULL);
   if(!pci) return true; // No proxy 
   switch (OBJ_obj2nid(pci->proxyPolicy->policyLanguage)) {
     case NID_Independent: { // No rights granted
       // Either such situation should be disallowed or 
-      // policy should be generated which granst no right for enything.
+      // policy should be generated which grants no right for anything.
       // First option is easier to implement so using it at least yet.
       logger.msg(DEBUG,"Independent proxy - no rights granted");
     }; break; 
@@ -68,11 +67,11 @@ static bool get_proxy_policy(X509* cert,DelegationMultiSecAttr* sattr) {
     }; break;
   };
   PROXY_CERT_INFO_EXTENSION_free(pci);
-#endif
   return result;
 }
 
 SecHandlerStatus DelegationCollector::Handle(Arc::Message* msg) const {
+  SecAttr* sattr_ = NULL;
   DelegationMultiSecAttr* sattr = NULL;
   try {
     MessagePayload* mpayload = msg->Payload();
@@ -81,8 +80,11 @@ SecHandlerStatus DelegationCollector::Handle(Arc::Message* msg) const {
     // Currently only TLS payloads are supported
     if(!tstream) return false;
     SecAttr* sattr_ = msg->Auth()->get("DELEGATION POLICY");
-    if(sattr_) sattr=dynamic_cast<DelegationMultiSecAttr*>(sattr_);
-    if(!sattr) sattr=new DelegationMultiSecAttr;
+    if(sattr_) sattr = dynamic_cast<DelegationMultiSecAttr*>(sattr_);
+    if(!sattr) {
+      sattr_ = NULL;
+      sattr = new DelegationMultiSecAttr;
+    }
     X509* cert = tstream->GetPeerCert();
     if (cert != NULL) {
       if(!get_proxy_policy(cert,sattr)) {
@@ -104,7 +106,7 @@ SecHandlerStatus DelegationCollector::Handle(Arc::Message* msg) const {
     if(!sattr_) msg->Auth()->set("DELEGATION POLICY",sattr);
     return true;
   } catch(std::exception&) { };
-  if(sattr) delete sattr;
+  if(!sattr_) delete sattr;
   return false;
 }
 
