@@ -72,6 +72,7 @@ namespace DataStaging {
        use_host_cert_for_remote_delivery(false),
        current_owner(GENERATOR),
        logger(log),
+       delete_log_destinations(true),
        perf_record(perf_log)
   {
     if (!logger) {
@@ -291,6 +292,17 @@ namespace DataStaging {
     return false;
   }
 
+  void DTR::clean_log_destinations(Arc::LogDestination* exclude) {
+    // This method makes sure log_destinations is in sync with the LogDestination
+    // objects inside logger. First clear internal list (not deleting objects)
+    log_destinations.clear();
+    // Then delete objects inside logger if requested
+    if (logger) {
+      if (delete_log_destinations) logger->deleteDestinations(exclude);
+      else logger->removeDestinations();
+    }
+  }
+
   std::list<DTRCallback*> DTR::get_callbacks(const std::map<StagingProcesses,std::list<DTRCallback*> >& proc_callback, StagingProcesses owner) {
     std::list<DTRCallback*> l;
     lock.lock();
