@@ -7,6 +7,7 @@
 #include <arc/Logger.h>
 #include <arc/URL.h>
 #include <arc/XMLNode.h>
+#include <arc/ArcConfigFile.h>
 
 #include "GMConfig.h"
 
@@ -46,6 +47,7 @@ public:
   bool get_use_host_cert_for_remote_delivery() const { return use_host_cert_for_remote_delivery; };
   Arc::LogLevel get_log_level() const { return log_level; };
   std::string get_dtr_log() const { return dtr_log; };
+  std::string get_dtr_central_log() const { return dtr_central_log; };
   std::string get_acix_endpoint() const { return acix_endpoint; };
 
 private:
@@ -99,6 +101,8 @@ private:
   std::string dtr_log;
   /// Log for performance metrics
   Arc::JobPerfLog perf_log;
+  /// Central log file for all DTR messages
+  std::string dtr_central_log;
 
   /// ACIX endpoint from which to find locations of cached files
   std::string acix_endpoint;
@@ -112,7 +116,7 @@ private:
   /// Read in params from XML config
   bool readStagingConf(const Arc::XMLNode& cfg);
   /// Read in params from ini config
-  bool readStagingConf(std::ifstream& cfile);
+  bool readStagingConf(Arc::ConfigFile& cfile);
   /// Convert parameter to integer with mimimum value of -1
   bool paramToInt(const std::string& param, int& value);
 
