@@ -261,8 +261,10 @@ bool job_diagnostics_mark_remove(const GMJob &job,const GMConfig &config) {
 
 bool job_diagnostics_mark_move(const GMJob &job,const GMConfig &config) {
   std::string fname1;
-  if (job.get_local() && !job.get_local()->sessiondir.empty()) fname1 = job.get_local()->sessiondir + sfx_diag;
-  else fname1 = job.SessionDir() + sfx_diag;
+  if (job.GetLocalDescription() && !job.GetLocalDescription()->sessiondir.empty())
+    fname1 = job.GetLocalDescription()->sessiondir + sfx_diag;
+  else
+    fname1 = job.SessionDir() + sfx_diag;
   std::string fname2 = config.ControlDir() + "/job." + job.get_id() + sfx_diag;
 
   std::string data;
@@ -317,7 +319,8 @@ bool job_mark_add(const std::string &fname,const std::string &content) {
 
 bool job_mark_put(const std::string &fname) {
   int h=open(fname.c_str(),O_WRONLY | O_CREAT,S_IRUSR | S_IWUSR);
-  if(h==-1) return false; close(h); return true;
+  if(h==-1) return false;
+  close(h); return true;
 }
 
 static bool job_mark_put(Arc::FileAccess& fa, const std::string &fname) {
@@ -347,6 +350,7 @@ static bool job_mark_remove(Arc::FileAccess& fa,const std::string &fname) {
 time_t job_mark_time(const std::string &fname) {
   struct stat st;
   if(lstat(fname.c_str(),&st) != 0) return 0;
+  if(st.st_mtime == 0) st.st_mtime = 1; // doomsday protection
   return st.st_mtime;
 }
 
@@ -362,6 +366,11 @@ bool job_errors_mark_put(const GMJob &job,const GMConfig &config) {
   return job_mark_put(fname) & fix_file_owner(fname,job) & fix_file_permissions(fname);
 }
 
+bool job_errors_mark_add(const GMJob &job,const GMConfig &config,const std::string &msg) {
+  std::string fname = config.ControlDir() + "/job." + job.get_id() + sfx_errors;
+  return job_mark_add(fname,msg) & fix_file_owner(fname,job) & fix_file_permissions(fname);
+}
+
 std::string job_errors_filename(const JobId &id, const GMConfig &config) {
   return config.ControlDir() + "/job." + id + sfx_errors;
 }
@@ -444,18 +453,13 @@ static job_state_t job_state_read_file(const std::string &fname,bool &pending) {
   } else {
     pending=false;
   };
-  for(int i = 0;states_all[i].name != NULL;i++) {
-    if(states_all[i].name == data) {
-      return states_all[i].id;
-    };
-  };
-  return JOB_STATE_UNDEFINED; /* broken file */
+  return GMJob::get_state(data.c_str());
 }
 
 static bool job_state_write_file(const std::string &fname,job_state_t state,bool pending) {
   std::string data;
   if (pending) data += "PENDING:";
-  data += states_all[state].name;
+  data += GMJob::get_state_name(state);
   return Arc::FileCreate(fname, data);
 }
 
@@ -665,6 +669,16 @@ std::string job_proxy_filename(const JobId &id, const GMConfig &config){
   return config.ControlDir() + "/job." + id + sfx_proxy;
 }
 
+bool job_proxy_write_file(const GMJob &job,const GMConfig &config,const std::string &cred) {
+  std::string fname = config.ControlDir() + "/job." + job.get_id() + sfx_proxy;
+  return Arc::FileCreate(fname, cred, 0, 0, S_IRUSR | S_IWUSR) & fix_file_owner(fname,job);
+}
+
+bool job_proxy_read_file(const JobId &id,const GMConfig &config,std::string &cred) {
+  std::string fname = config.ControlDir() + "/job." + id + sfx_proxy;
+  return Arc::FileRead(fname, cred, 0, 0);
+}
+
 bool job_clean_finished(const JobId &id,const GMConfig &config) {
   std::string fname;
   fname = config.ControlDir()+"/job."+id+".proxy.tmp"; remove(fname.c_str());
@@ -676,8 +690,10 @@ bool job_clean_deleted(const GMJob &job,const GMConfig &config,std::list<std::st
   std::string id = job.get_id();
   job_clean_finished(id,config);
   std::string session;
-  if(job.get_local() && !job.get_local()->sessiondir.empty()) session = job.get_local()->sessiondir;
-  else session = job.SessionDir();
+  if(job.GetLocalDescription() && !job.GetLocalDescription()->sessiondir.empty())
+    session = job.GetLocalDescription()->sessiondir;
+  else
+    session = job.SessionDir();
   std::string fname;
   fname = config.ControlDir()+"/job."+id+sfx_proxy; remove(fname.c_str());
   fname = config.ControlDir()+"/"+subdir_new+"/job."+id+sfx_restart; remove(fname.c_str());
