@@ -81,7 +81,7 @@ void DTRGenerator::thread() {
       processReceivedDTR(*it_dtrs);
       event_lock.lock();
       // delete DTR LogDestinations
-      (*it_dtrs)->get_logger()->deleteDestinations();
+      (*it_dtrs)->clean_log_destinations(central_dtr_log);
       it_dtrs = dtrs_received.erase(it_dtrs);
     }
 
@@ -114,7 +114,7 @@ void DTRGenerator::thread() {
   while (it_dtrs != dtrs_received.end()) {
     processReceivedDTR(*it_dtrs);
     // delete DTR LogDestinations
-    (*it_dtrs)->get_logger()->deleteDestinations();
+    (*it_dtrs)->clean_log_destinations(central_dtr_log);
     it_dtrs = dtrs_received.erase(it_dtrs);
   }
   run_condition.signal();
@@ -126,6 +126,7 @@ DTRGenerator::DTRGenerator(const GMConfig& config,
                            void* kicker_arg) :
     generator_state(DataStaging::INITIATED),
     config(config),
+    central_dtr_log(NULL),
     staging_conf(config),
     info(config),
     kicker_func(kicker_func),
@@ -148,6 +149,11 @@ DTRGenerator::DTRGenerator(const GMConfig& config,
   // If those destinations appear again, add overwrite=yes
   readDTRState(dtr_log);
 
+  // Central DTR log if configured
+  if (!staging_conf.get_dtr_central_log().empty()) {
+    central_dtr_log = new Arc::LogFile(staging_conf.get_dtr_central_log());
+  }
+
   // Processing limits
   scheduler->SetSlots(staging_conf.max_processor,
                       staging_conf.max_processor,
@@ -610,7 +616,7 @@ bool DTRGenerator::processReceivedJob(GMJob& job) {
     JobLocalDescription job_desc;
     if(job_local_read_file(jobid, config, job_desc)) {
       if(!job_desc.delegationid.empty()) {
-        ARex::DelegationStores* delegs = config.Delegations();
+        ARex::DelegationStores* delegs = config.GetDelegations();
         if(delegs) {
           DelegationStore& deleg = delegs->operator[](config.DelegationDir());
           std::string fname = deleg.FindCred(job_desc.delegationid, job_desc.DN);
@@ -926,6 +932,9 @@ bool DTRGenerator::processReceivedJob(GMJob& job) {
     dest->setReopen(true);
     dest->setFormat(Arc::MediumFormat);
     dtr_log->addDestination(*dest);
+    if (central_dtr_log) {
+      dtr_log->addDestination(*central_dtr_log);
+    }
 
     // create DTR and send to Scheduler
     DataStaging::DTR_ptr dtr(new DataStaging::DTR(source, destination, usercfg, jobid, job.get_user().get_uid(), dtr_log));
@@ -1206,7 +1215,7 @@ int DTRGenerator::user_file_exists(FileData &dt,
       if (l == 0) break;
       crc.add(buffer, l);
     }
-    close(h);
+    if (h != -1) close(h);
     if (fa) fa->fa_close();
     delete fa;
     crc.end();
