@@ -16,6 +16,14 @@
    @author: Will Rogers
 '''
 
+# It's possible for SSM to be used without SSL, and the ssl module isn't in the
+# standard library until 2.6, so this makes it safe for earlier Python versions.
+try:
+    import ssl
+except ImportError:
+    # ImportError is raised later on if SSL is actually requested.
+    ssl = None
+
 import crypto
 from dirq.QueueSimple import QueueSimple
 from dirq.queue import Queue
@@ -42,7 +50,8 @@ class Ssm2Exception(Exception):
     '''
     pass
 
-class Ssm2(object):
+
+class Ssm2(stomp.ConnectionListener):
     '''
     Minimal SSM implementation.
     '''
@@ -95,7 +104,7 @@ class Ssm2(object):
             raise Ssm2Exception('Cert and key don\'t match.')
         # check the server certificate provided
         if enc_cert is not None:
-            log.info('Messages will be encrypted using %s' % enc_cert)
+            log.info('Messages will be encrypted using %s', enc_cert)
             if not os.path.isfile(self._enc_cert):
                 raise Ssm2Exception('Specified certificate file does not exist: %s.' % self._enc_cert)
             if verify_enc_cert:
@@ -157,13 +166,13 @@ class Ssm2(object):
                                'signer':signer, 
                                'empaid': headers['empa-id']})
         except OSError, e:
-            log.error('Failed to read or write file: %s' % e)
+            log.error('Failed to read or write file: %s', e)
         
     def on_error(self, unused_headers, body):
         '''
         Called by stomppy when an error frame is received.
         '''
-        log.warn('Error message received: %s' % body)
+        log.warn('Error message received: %s', body)
         raise Ssm2Exception()
     
     def on_connected(self, unused_headers, unused_body):
@@ -210,21 +219,21 @@ class Ssm2(object):
             try:
                 text = crypto.decrypt(text, self._cert, self._key)
             except crypto.CryptoException, e:
-                log.error('Failed to decrypt message: %s' % e)
+                log.error('Failed to decrypt message: %s', e)
                 return None, None
         
         # always signed
         try:
             message, signer = crypto.verify(text, self._capath, self._check_crls)
         except crypto.CryptoException, e:
-            log.error('Failed to verify message: %s' % e)
+            log.error('Failed to verify message: %s', e)
             return None, None
         
         if signer not in self._valid_dns:
-            log.error('Message signer not in the valid DNs list: %s' % signer)
+            log.error('Message signer not in the valid DNs list: %s', signer)
             return None, signer
         else:
-            log.info('Valid signer: %s' % signer)
+            log.info('Valid signer: %s', signer)
             
         return message, signer
         
@@ -271,10 +280,10 @@ class Ssm2(object):
         '''
         Send all the messages in the outgoing queue.
         '''
-        log.info('Found %s messages.' % self._outq.count())
+        log.info('Found %s messages.', self._outq.count())
         for msgid in self._outq:
             if not self._outq.lock(msgid):
-                log.warn('Message queue was locked. %s will not be sent.' % msgid)
+                log.warn('Message was locked. %s will not be sent.', msgid)
                 continue
 
             text = self._outq.get(msgid)
@@ -290,6 +299,13 @@ class Ssm2(object):
             self._last_msg = None
             self._outq.remove(msgid)
 
+        log.info('Tidying message directory.')
+        try:
+            # Remove empty dirs and unlock msgs older than 5 min (default)
+            self._outq.purge()
+        except OSError, e:
+            log.warn('OSError raised while purging message queue: %s', e)
+
     ############################################################################
     # Connection handling methods
     ############################################################################
@@ -300,15 +316,35 @@ class Ssm2(object):
         but don't try to start the connection.
         '''
         if self._use_ssl:
+            if ssl is None:
+                raise ImportError("SSL connection requested but the ssl module "
+                                  "wasn't found.")
             log.info('Connecting using SSL...')
-            
-        self._conn = stomp.Connection([(host, port)], 
-                                      use_ssl=self._use_ssl,
-                                      user = self._user,
-                                      passcode = self._pwd,
-                                      ssl_key_file = self._key,
-                                      ssl_cert_file = self._cert)
-        
+
+            try:
+                # Compatible with stomp.py >= 3.0.4
+                self._conn = stomp.Connection([(host, port)],
+                                              use_ssl=self._use_ssl,
+                                              user=self._user,
+                                              passcode=self._pwd,
+                                              ssl_key_file=self._key,
+                                              ssl_cert_file=self._cert,
+                                              ssl_version=ssl.PROTOCOL_SSLv23)
+            except TypeError:
+                # For stomp.py <= 3.0.3, override ssl.PROTOCOL_SSLv3 and then
+                # try to set up the connection again below.
+                ssl.PROTOCOL_SSLv3 = ssl.PROTOCOL_SSLv23
+
+        if self._conn is None:
+            # If _conn is None then either SSL wasn't requested or trying to
+            # set ssl_version failed.
+            self._conn = stomp.Connection([(host, port)],
+                                          use_ssl=self._use_ssl,
+                                          user=self._user,
+                                          passcode=self._pwd,
+                                          ssl_key_file=self._key,
+                                          ssl_cert_file=self._cert)
+
         # You can set this in the constructor but only for stomppy version 3.
         # This works for stomppy 3 but doesn't break stomppy 2.
         self._conn.__reconnect_attempts_max = 1
@@ -331,10 +367,10 @@ class Ssm2(object):
                 break
             except ConnectFailedException, e:
                 # ConnectFailedException doesn't provide a message.
-                log.warn('Failed to connect to %s:%s.' % (host, port))
+                log.warn('Failed to connect to %s:%s.', host, port)
             except Ssm2Exception, e:
-                log.warn('Failed to connect to %s:%s: %s' % (host, port, str(e)))
-                
+                log.warn('Failed to connect to %s:%s: %s', host, port, e)
+
         if not self.connected:
             raise Ssm2Exception('Attempts to start the SSM failed.  The system will exit.')
 
@@ -378,12 +414,12 @@ class Ssm2(object):
         self._conn.connect(wait = True)
         
         if self._dest is not None:
-            log.info('Will send messages to: %s' % self._dest)
-             
+            log.info('Will send messages to: %s', self._dest)
+
         if self._listen is not None:
             self._conn.subscribe(destination=self._listen, ack='auto')
-            log.info('Subscribing to: %s' % self._listen)
-            
+            log.info('Subscribing to: %s', self._listen)
+
         i = 0
         while not self.connected:
             time.sleep(0.1)
@@ -400,7 +436,7 @@ class Ssm2(object):
         if it is not ended.
         '''
         try:
-            self._conn.disconnect()
+            self._conn.stop()  # Same as diconnect() but waits for thread exit
         except (stomp.exception.NotConnectedException, socket.error):
             self._conn = None
         except AttributeError:
@@ -420,8 +456,8 @@ class Ssm2(object):
                 f.write('\n')
                 f.close()
             except IOError, e:
-                log.warn('Failed to create pidfile %s: %s' % (self._pidfile, e))
-                
+                log.warn('Failed to create pidfile %s: %s', self._pidfile, e)
+
         self.handle_connect()
         
     def shutdown(self):
@@ -434,9 +470,9 @@ class Ssm2(object):
                 if os.path.exists(self._pidfile):
                     os.remove(self._pidfile)
                 else:
-                    log.warn('pidfile %s not found.' % self._pidfile)
+                    log.warn('pidfile %s not found.', self._pidfile)
             except IOError, e:
-                log.warn('Failed to remove pidfile %s: %e' % (self._pidfile, e))
+                log.warn('Failed to remove pidfile %s: %e', self._pidfile, e)
                 log.warn('SSM may not start again until it is removed.')
         
         
